var ConvertXmlToJson = (function () {
    'use strict';
    /**
     * @description Convertis un XML en JSON compatible cytoscape.
     * @param {Element} xml The XML DOM object.
     * @typedef ConvertXmlToJson Convertis un graphml en JSON. Nécessite un argument de type Element (DOM Element).
     */
    function ConvertXmlToJson(xml) {
        /**
         * @description Le XML passé en paramètre.
         * @private
         */
        var _xml = xml;
        /**
         * @description Tableau contenant le JSON correspondant aux XML.
         * @type Object
         * @private
         */
        var jsonObjectTableau = [];
        /**
         * @description Donne le JSON.
         * @returns {Object}
         */
        this.getJSON = function () {
            return jsonObjectTableau;
        };
        /**
         * @description Donne la taille du tableau de JSON.
         * @returns {Number}
         */
        this.getJsonSize = function () {
            return jsonObjectTableau.length;
        };
        /**
         * @description Donne le XML.
         * @returns {Element} L'objet XML DOM brut.
         */
        this.getXML = function () {
            return _xml;
        };
        /**
         * @description Récupère tous les 'node' du XML et les introduits dans le 'jsonObjectTableau'.
         */
        function recuperationDesNodes() {
            $(_xml).find("*").filter(function () {
                /* Récupération de tous les éléments 'node' du XML. */
                return this.nodeName.toLowerCase().match(/node$/) !== null;
            }).each(function () {
                /* Création de l'objet JSON à partir du l'élément XML actuellement en cours de parcours. Récupération de l'id et assignation d'un group. */
                var jsonNodeObject = {data: {id: $(this).attr("id")}, group: elementGroup.node};
                /* Altération de l'objet, par le biais d'une fonction, pour ajout des différentes 'data'. */
                getNodeData($(this), jsonNodeObject);
                /* Ajout de l'objet dans le jsonObjectTableau. */
                jsonObjectTableau.push(jsonNodeObject);
            });
        }
        /**
         * @description Récupère les différents paramètre du node passé en paramètre.
         * @param {Element} node Les éléments node du XML.
         * @param {Object} jsonNodeObject Les éléments node au format JSON compatible cytoscape.
         */
        function getNodeData(node, jsonNodeObject) {
            $(node).find("*").filter(function () {
                /* Récupération de chaque élément 'data' du node. */
                return this.nodeName.toLowerCase().match(/data$/) !== null;
            }).each(function () {
                /* Récupération de toutes les data du node sous forme de clé/valeur. Attribution du tous dans l'attribut data de l'objet JSON.  */
                jsonNodeObject.data[($(this).attr("key").toLowerCase() === "label" ? "class" : $(this).attr("key").toLowerCase())] = ($(this).text() !== "" ? $(this).text() : undefined);          // Création d'une nouvelle propriété de l'objet jsonNodeObject qui correspond à l'attribut 'key' de data et ajout de sa valeur.
            });
        }
        /**
         * @description Récupère tous les 'edge' du XML et les introduits dans le 'jsonObjectTableau'.
         */
        function recuperationDesEdges() {
            $(_xml).find("*").filter(function () {
                /* Récupération de tous les élément 'edge' du XML. */
                return this.nodeName.toLowerCase().match(/edge$/) !== null;
            }).each(function () {
                /* Création de l'objet edge au format JSON. Récupération de la source et de la target de l'élément parcourut et assignation du group. */
                var jsonEdgeObject = {data: {source: $(this).attr("source"), target: $(this).attr("target")}, group : elementGroup.edge};
                /* Altération de l'objet, par le biais d'une fonction, pour ajout des data supplémentaires. */
                getEdgeData($(this), jsonEdgeObject);
                /* Ajout de l'objet dans jsonObjectTableau */
                jsonObjectTableau.push(jsonEdgeObject);
            });
        }/**
         * @description Récupère les différents paramètre du edge passé en paramètre.
         * @param {Element} edge Les éléments edge du XML.
         * @param {Object} jsonEdgeObject Les éléments edge au format JSON compatible cytoscape.
         */
        function getEdgeData(edge, jsonEdgeObject) {
            $(edge).find("*").filter(function () {
                /* Récupération de tous les élément 'data' du edge. */
                return this.nodeName.toLowerCase().match(/data$/) !== null;
            }).each(function () {
                /* Parcours de tous les éléments data du edge ; récupération de clé/valeur à partir de l'attribut 'key' et de la valeur du texte. */
                jsonEdgeObject.data[$(this).attr("key").toLowerCase()] = ($(this).text() !== "" ? $(this).text() : undefined);
            });
        }

        recuperationDesNodes();
        recuperationDesEdges();
    }

    return ConvertXmlToJson;
}());